@@ -1,5 +1,5 @@
 #
-# Copyright (C) 2014-2015 by Tomasz Konojacki
+# Copyright (C) 2014-2016 by Tomasz Konojacki
 #
 # This library is free software; you can redistribute it and/or modify
 # it under the same terms as Perl itself, either Perl version 5.18.2 or,
@@ -12,9 +12,11 @@ use strict;
 use warnings;
 
 use IO::File;
+use File::AtomicWrite;
+
 use Carp;
 
-our $VERSION = '1.01';
+our $VERSION = '1.2';
 
 sub __init {
     my $self = shift;
@@ -24,13 +26,16 @@ sub __init {
         ($self->{readonly} ? O_RDONLY : O_RDWR) |
         ($self->{create_file} ? O_CREAT : 0)
     ) or croak sprintf('Unable to open file "%s".', $self->{filename});
-
     # we utf-8, baby
     $self->{_fh}->binmode(':utf8');
 
-    # init cache
     $self->{_cache} = {};
 
+    $self->__read;
+
+    close $self->{_fh};
+    undef $self->{_fh};
+
     return 1;
 }
 
@@ -97,18 +102,19 @@ sub __write {
 
     my $cache = $self->{_cache};
 
-    # clear the file before we write to it.
-    $self->{_fh}->truncate(0);
-    $self->{_fh}->seek(0, 0);
+    my $faw = File::AtomicWrite->new({
+        file => $self->{filename},
+        binmode_layer => ':utf8'
+    });
 
+    my $fh = $faw->fh;
     for my $item (@{ $cache->{structure} }) {
         next if $item->{deleted};
 
-        $self->{_fh}->print($item->{value}, "\n");
+        $fh->print($item->{value}, "\n");
     }
 
-    # Due to flushing, writes are reliable even if user forgets to untie hash.
-    $self->{_fh}->flush;
+    $faw->commit;
 
     return;
 }
@@ -130,7 +136,6 @@ sub TIEHASH {
     bless $self, $class;
 
     $self->__init;
-    $self->__read;
 
     return $self;
 }
@@ -230,14 +235,6 @@ sub UNTIE {
     return
 }
 
-sub DESTROY {
-    my $self = shift;
-
-    $self->{_fh}->close;
-
-    return
-}
-
 'thank you based god';
 
 __END__
@@ -325,17 +322,17 @@ L<Tie::Config>
 
 =head1 FOSSIL REPOSITORY
 
-DBD::IngresII Fossil repository is hosted at xenu.tk:
+Tie::ConfigFile Fossil repository is hosted at my own server:
 
-    http://code.xenu.tk/repos.cgi/tie-configfile
+    http://code.xenu.pl/repos.cgi/tie-configfile
 
 =head1 AUTHOR
 
-    Tomasz Konojacki <me@xenu.tk>
+    Tomasz Konojacki <me@xenu.pl>
 
 =head1 COPYRIGHT AND LICENSE
 
-Copyright (C) 2014-2015 by Tomasz Konojacki
+Copyright (C) 2014-2016 by Tomasz Konojacki
 
 This library is free software; you can redistribute it and/or modify
 it under the same terms as Perl itself, either Perl version 5.18.2 or,
